UNH EXS Contribution Guide
==========================

All contributions should be submitted via merge request to the
[upstream GitLab project](https://oss.iol.unh.edu/research/unh-exs).

Each commit within the merge request must be signed off by appending a
line like this to the end of the commit message using your full real
name and e-mail address:

    Signed-off-by: Joe Developer <joe@example.com>

indicating that you agree to these terms (as excerpted from the Linux
kernel [submission guidelines][1]):

> ### Developer's Certificate of Origin 1.1
>
> By making a contribution to this project, I certify that:
>
> (a) The contribution was created in whole or in part by me and I
>     have the right to submit it under the open source license
>     indicated in the file; or
>
> (b) The contribution is based upon previous work that, to the best
>     of my knowledge, is covered under an appropriate open source
>     license and I have the right under that license to submit that
>     work with modifications, whether created in whole or in part
>     by me, under the same open source license (unless I am
>     permitted to submit under a different license), as indicated
>     in the file; or
>
> (c) The contribution was provided directly to me by some other
>     person who certified (a), (b) or (c) and I have not modified
>     it.
>
> (d) I understand and agree that this project and the contribution
>     are public and that a record of the contribution (including all
>     personal information I submit with it, including my sign-off) is
>     maintained indefinitely and may be redistributed consistent with
>     this project or the open source license(s) involved.

You must agree to license your contribution as dual BSD/GPLv2 in
perpetuity (see the COPYING file for the full details of the
licenses).

You can automatically append your sign-off to a commit message by
including the `-s` flag to `git commit`.

Please follow the established git [commit guidelines][2] when
structuring your commit messages.  In particular, keep the subject line
less than 60-70 characters; put any other explanation in subsequent
paragraphs separated by blank lines.  The subject line must also be
separated from anything else, including your sign off, by a blank line.

All changes must match the coding style, which is based on the Linux
kernel [style guide][3].

To submit a merge request:

(1) If you have not already, you will need to create an account on the
GitLab instance, and upload your SSH public key.

(2) If you have not yet forked the UNH EXS project, you should email
it@iol.unh.edu to request the permission to fork a project, and then
fork the UNH EXS project.

(3) Set up your git client to push to your fork.  You can make a fresh
clone with:

    git clone git@oss.iol.unh.edu:<your-username>/unh-exs.git

Or point an existing repository at your fork with:

    git remote rm origin
    git remote add origin git@oss.iol.unh.edu:<your-username>/unh-exs.git
    git fetch origin

To be able to pull changes from the upstream repository:

    git remote add upstream git@oss.iol.unh.edu:research/unh-exs.git
    git fetch upstream

(4) Prepare a branch with your commits, rebase if needed, and then push
it.

    $ git checkout -b <branch-name>
    $ git rebase -i upstream/master
    $ git push origin

Your branch should be made against the latest upstream release
(preferred), or against the latest upstream master if your patch would
otherwise not apply cleanly or your branch depends on work that has been
done since the most recent release.

(5) Create a merge request in the GitLab Web UI with a description of the
changes.  Running git push gives you a message with a URL to
create/update the merge request.

If you need to make changes to your pull request, you can rebase and
force-push to your branch as many times as you need.  The merge request
will be updated automatically.

[1]: https://www.kernel.org/doc/Documentation/SubmittingPatches
[2]: https://git-scm.com/book/ch5-2.html#Commit-Guidelines
[3]: https://www.kernel.org/doc/Documentation/CodingStyle
