/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_LISTEN_SETUP_H
#define _EXS_LISTEN_SETUP_H

#include "../result.h"

#define LISTEN_TESTS 5

static char *all_listen_tests[LISTEN_TESTS] = {
	"1  : EPERM                   : exs_listen(fd, 10) [no exs_init]",
	"2  : EBADF                   : exs_listen(BAD_FD, 10): ",
	"3  : EINVAL                  : exs_listen(CONNECTED_FD, 10)",
	"4  : EINVAL                  : exs_listen(fd, -1)",
	"5  : OK_EXS_LISTEN           : exs_listen(fd, 10)"
};

static int list_listen_tests(int select)
{
	if (select < 0 || select > LISTEN_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			LISTEN_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < LISTEN_TESTS; i++) {
			fprintf(stdout, "%s\n", all_listen_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_listen_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_listen(param * p)
{
	char *name = "ok_exs_listen";

	struct addrinfo *info;
	struct addrinfo hints;
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = p->family;
	hints.ai_socktype = p->socktype;
	hints.ai_protocol = p->protocol;
	hints.ai_flags = AI_PASSIVE;

	int r = getaddrinfo(p->port_name, p->port_num, &hints, &info);

	int ret = exs_listen(p->fd, 10);

	if (ret > 0) {
		print_error(name);
		return -1;
	}
	return 0;
}

#endif
